@@ -19,9 +19,8 @@
 # You should have received a copy of the GNU General Public License
 # along with Project Hamster.  If not, see <http://www.gnu.org/licenses/>.
 
-
+import logging
 import datetime as dt
-import os.path
 
 import pygtk
 pygtk.require("2.0")
@@ -44,7 +43,7 @@ from edit_activity import CustomFactController
 from stats import StatsViewer
 from about import show_about
 from preferences import PreferencesEditor
-
+import widgets
 import idle
 
 import pango
@@ -127,8 +126,8 @@ class PanelButton(gtk.ToggleButton):
             else:
                 label = "%s %s" % (self.activity, self.duration)
         
-        label = stuff.escape_pango(label)
-        label = '<span gravity=\"south\">' + label + '</span>'
+        label = '<span gravity="south">%s</span>' % stuff.escape_pango(label)
+        self.label.set_markup("") #clear - seems to fix the warning
         self.label.set_markup(label)
 
     def get_pos(self):
@@ -230,12 +229,24 @@ class HamsterApplet(object):
         self._gui = stuff.load_ui_file("applet.ui")
         self.window = self._gui.get_object('hamster-window')
 
-        
-        # build the menu
-        self.init_dropdown()
+        self.new_name = widgets.ActivityEntry()
+        widgets.add_hint(self.new_name, _("Time and Name"))
+        self.get_widget("new_name_box").add(self.new_name)
+        self.new_name.connect("changed", self.on_activity_text_changed)
 
-        # init today's tree
-        self.setup_activity_tree()
+        self.new_tags = widgets.TagsEntry()
+        widgets.add_hint(self.new_tags, _("Tags or Description"))
+        self.get_widget("new_tags_box").add(self.new_tags)
+
+        self.tag_box = widgets.TagBox(interactive = False)
+        self.get_widget("tag_box").add(self.tag_box)
+
+        self.treeview = widgets.FactTree()
+        self.treeview.connect("key-press-event", self.on_todays_keys)
+        self.treeview.connect("edit-clicked", self._open_edit_activity)
+        self.treeview.connect("row-activated", self.on_today_row_activated)
+        
+        self.get_widget("today_box").add(self.treeview)
 
         # DBus Setup
         try:
@@ -248,8 +259,10 @@ class HamsterApplet(object):
             runtime.dispatcher.add_handler('active_changed', self.on_idle_changed)
 
         except dbus.DBusException, e:
-            print "can't init dbus: %s" % e
+            logging.error("Can't init dbus: %s" % e)
     
+        self.day_start = self.config.get_day_start()
+
         # Load today's data, activities and set label
         self.last_activity = None
         self.load_day()
@@ -279,6 +292,7 @@ class HamsterApplet(object):
         runtime.dispatcher.add_handler('gconf_notify_on_idle_changed', self.on_notify_on_idle_changed)
         self.notify_on_idle = self.config.get_notify_on_idle()
         
+        runtime.dispatcher.add_handler('gconf_on_day_start_changed', self.on_day_start_changed)
         
         # init nagging timeout
         if PYNOTIFY:
@@ -287,185 +301,6 @@ class HamsterApplet(object):
             self.on_notify_interval_changed(None, self.config.get_notify_interval())
 
 
-
-    def setup_activity_tree(self):
-        self.treeview = self._gui.get_object('today')
-        # ID, Time, Name, Duration, Date, Description, Category
-        self.treeview.set_model(gtk.ListStore(int, str, str, str, str, str, str, gobject.TYPE_PYOBJECT))
-
-        self.treeview.append_column(gtk.TreeViewColumn("Time",
-                                                       gtk.CellRendererText(),
-                                                       text=2))
-        self.treeview.append_column(stuff.ActivityColumn(name=1,
-                                                         description=5,
-                                                         category=6))
-        
-        duration_cell = gtk.CellRendererText()
-        duration_cell.set_property("xalign", 1)
-        timeColumn = gtk.TreeViewColumn(_("Duration"), duration_cell, text=3)
-        
-        self.treeview.append_column(timeColumn)
-
-        edit_cell = gtk.CellRendererPixbuf()
-        edit_cell.set_property("stock_id", "gtk-edit")
-        edit_cell.set_property("mode", gtk.CELL_RENDERER_MODE_ACTIVATABLE)
-        self.edit_column = gtk.TreeViewColumn("", edit_cell)
-        self.treeview.append_column(self.edit_column)
-
-
-    def init_dropdown(self):
-        # set up drop down menu
-        self.activity_combo = self._gui.get_object('activity_combo')
-        self.activity_combo.child.connect('activate', self.on_activity_entered)
-        self.activity_combo.child.connect('key-press-event', self.on_activity_list_key_pressed)
-        self.activity_combo.child.connect('changed', self.on_activity_entry_changed)
-
-        self.activity_combo.set_model(gtk.ListStore(gobject.TYPE_STRING,
-                                                   gobject.TYPE_STRING,
-                                                   gobject.TYPE_STRING))
-        self.activity_combo.set_text_column(2)
-        self.activity_combo.clear()
-
-
-        activity_cell = gtk.CellRendererText()
-        self.activity_combo.pack_start(activity_cell, True)
-        self.activity_combo.add_attribute(activity_cell, 'text', 0)
-
-        category_cell = stuff.CategoryCell()  
-        self.activity_combo.pack_start(category_cell, False)
-        self.activity_combo.add_attribute(category_cell, 'text', 1)
-        
-
-        #now set up completion
-        self.completion = gtk.EntryCompletion()
-        #fillable, activity_name, category
-        self.completion.set_model(gtk.ListStore(gobject.TYPE_STRING,
-                                                gobject.TYPE_STRING,
-                                                gobject.TYPE_STRING))
-        self.completion.set_minimum_key_length(1)
-        self.completion.set_inline_completion(True)
-        self.completion.set_popup_set_width(False)
-        self.completion.set_popup_single_match(False)
-        self.completion.set_text_column(0)
-        self.activity_combo.child.set_completion(self.completion)
-        
-        self.refresh_dropdown()
-
-
-    def refresh_dropdown(self):        
-        self.all_activities = runtime.storage.get_autocomplete_activities()
-        self.all_categories = runtime.storage.get_category_list()
-        self.eds_tasks = eds.get_eds_tasks()
-
-        #add evolution tasks to dropdown, yay!
-        for activity in self.eds_tasks:
-            self.all_activities.append(activity)
-
-        #now populate the menu - contains only categorized entries
-        store = self.activity_combo.get_model()
-        store.clear()
-
-        categorized_activities = runtime.storage.get_sorted_activities()
-        for activity in categorized_activities:
-            activity_category = activity['name']
-            if activity['category']:
-                activity_category += "@%s" % activity['category']
-
-            store.append([activity['name'],
-                          activity['category'],
-                          activity_category])
-
-
-    def on_match_selected(self, completion, model, iter):
-        entry = completion.get_entry()
-        activity = stuff.parse_activity_input(entry.get_text())
-        
-        fillable = model.get_value(iter, 0)
-        
-        res = ""
-        
-        if activity.start_time:
-            res += "%s" % activity.start_time.strftime("%H:%M")
-            if activity.end_time:
-                res += "-%s" % activity.start_time.strftime("%H:%M")
-        
-            res += " "
-            #strip time
-            fillable = fillable[fillable.find(" ")+1:]
-        
-        res += fillable
-        
-        entry.set_text(res)
-        entry.set_position(len(res))
-        return True
-    
-    def redo_autocomplete(self):
-        """parses input and generates autocomplete according to what has
-        to be completed now"""
-        #TODO turn this whole thing into a widget
-        store = self.completion.get_model()
-        store.clear()
-        
-        input_text = self.activity_combo.child.get_text()
-        
-        if not input_text:
-            return
-        
-        
-        entry = self.activity_combo.child
-        parsed_activity = stuff.parse_activity_input(entry.get_text())
-
-        if input_text.find("@") > 0:
-            key = input_text[input_text.find("@")+1:].lower()
-            for category in self.all_categories:
-                if key in category['name'].lower():
-                    fillable = (input_text[:input_text.find("@") + 1] + category['name']).lower()
-                    store.append([fillable, category['name'], fillable])
-
-        else:
-            for activity in self.all_activities:
-                fillable = activity['name']
-                if activity['category']:
-                    fillable += "@%s" % activity['category']
-                    
-                if parsed_activity.start_time:
-                    fillable = entry.get_text()[:entry.get_text().find(" ")+1] + fillable
-    
-                store.append([fillable, activity['name'], activity['category']])
-
-                
-        activity = stuff.parse_activity_input(input_text)
-        self.completion.clear()
-        if activity.start_time:
-            bgcolor = gtk.Style().bg[gtk.STATE_NORMAL].to_string()
-            time_cell = gtk.CellRendererPixbuf()
-            time_cell.set_property("icon-name", "appointment-new")
-            time_cell.set_property("cell-background", bgcolor)
-            
-            self.completion.pack_start(time_cell, False)
-
-            time_cell = gtk.CellRendererText()
-            time = activity.start_time.strftime("%H:%M")
-            if activity.end_time:
-                time += "-%s" % activity.end_time.strftime("%H:%M")
-                
-            time_cell.set_property("text", time)
-            time_cell.set_property("scale", 0.8)
-            time_cell.set_property("cell-background", bgcolor)
-            self.completion.pack_start(time_cell, False)
-
-        activity_cell = gtk.CellRendererText()
-        self.completion.pack_start(activity_cell, True)
-        self.completion.add_attribute(activity_cell, 'text', 1)
-        #self.completion.set_property("text-column", 2)
-
-        if input_text.find("@") == -1:
-            #no need for category cell when we populate categories
-            category_cell = stuff.CategoryCell()  
-            self.completion.pack_start(category_cell, False)
-            self.completion.add_attribute(category_cell, 'text', 2)
-        
-
     """UI functions"""
     def refresh_hamster(self):
         """refresh hamster every x secs - load today, check last activity etc."""        
@@ -486,14 +321,44 @@ class HamsterApplet(object):
                                stuff.format_duration(duration, False))
             self.button.set_text(self.last_activity['name'],
                                  stuff.format_duration(duration, False))
-            
-            self._gui.get_object('stop_tracking').set_sensitive(1);
         else:
             label = "%s" % _(u"No activity")
             self.button.set_text(label, None)
-            self._gui.get_object('stop_tracking').set_sensitive(0);
+            
+        self.set_last_activity()
+
+        
+    def set_last_activity(self):
+        activity = runtime.storage.get_last_activity()
+        self.get_widget("stop_tracking").set_sensitive(activity != None)
         
         
+        if activity:
+            self.get_widget("switch_activity").show()
+            self.get_widget("start_tracking").hide()
+            
+            delta = dt.datetime.now() - activity['start_time']
+            duration = delta.seconds /  60
+            
+            self.get_widget("last_activity_duration").set_text(stuff.format_duration(duration) or _("Just started"))
+            self.get_widget("last_activity_name").set_text(activity['name'])
+            if activity['category'] != _("Unsorted"):
+                self.get_widget("last_activity_category") \
+                    .set_text(" - %s" % activity['category'])
+
+            self.get_widget("last_activity_description").set_text(activity['description'] or "")
+
+            self.tag_box.draw(activity["tags"])
+        else:
+            self.get_widget("switch_activity").hide()
+            self.get_widget("start_tracking").show()
+
+            self.get_widget("last_activity_name").set_text(_("No activity"))
+            self.get_widget("last_activity_duration").set_text("")
+            self.get_widget("last_activity_category").set_text("")
+            self.tag_box.draw([])
+            self.get_widget("last_activity_description").set_text("")
+         
     def check_user(self):
         if not self.notify_interval: #no interval means "never"
             return
@@ -524,44 +389,45 @@ class HamsterApplet(object):
         """sets up today's tree and fills it with records
            returns information about last activity"""
         #today is 5.5 hours ago because our midnight shift happens 5:30am
-        today = (dt.datetime.now() - dt.timedelta(hours=5, minutes=30)).date()
+        today = (dt.datetime.now() - dt.timedelta(hours = self.day_start.hour,
+                                                  minutes = self.day_start.minute)).date()
 
         self.last_activity = runtime.storage.get_last_activity()
 
-        fact_store = self.treeview.get_model()
-        fact_store.clear()
+        self.treeview.clear()
+
         facts = runtime.storage.get_facts(today)
         
+        if len(facts) > 10:
+            self._gui.get_object("today_box").set_size_request(-1, 250)
+            self._gui.get_object("today_box").set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
+        else:
+            self._gui.get_object("today_box").set_size_request(-1, -1)
+            self._gui.get_object("today_box").set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_NEVER)
+            
         by_category = {}
-        
         for fact in facts:
             duration = 24 * 60 * fact["delta"].days + fact["delta"].seconds / 60
             by_category[fact['category']] = \
                           by_category.setdefault(fact['category'], 0) + duration
-
-            fact_store.append([fact['id'],
-                               stuff.escape_pango(fact['name']), 
-                               fact["start_time"].strftime("%H:%M"), 
-                               "%s" % stuff.format_duration(duration),
-                               fact["start_time"].strftime("%H:%M"),
-                               stuff.escape_pango(fact["description"]),
-                               stuff.escape_pango(fact["category"]),
-                               fact])
+            self.treeview.add_fact(fact)
 
         
         if not facts:
-            self._gui.get_object("todays_scroll").hide()
+            self._gui.get_object("today_box").hide()
             self._gui.get_object("fact_totals").set_text(_("No records today"))
         else:
-            self._gui.get_object("todays_scroll").show()
+            self._gui.get_object("today_box").show()
             
             total_strings = []
             for category in by_category:
                 # listing of today's categories and time spent in them
+                duration = "%.1f" % (by_category[category] / 60.0)
                 total_strings.append(_("%(category)s: %(duration)s") % \
                         ({'category': category,
-                          'duration': _("%.1fh") %
-                                               (by_category[category] / 60.0)}))
+                          #duration in main drop-down per category in hours
+                          'duration': _("%sh") % duration 
+                          }))
 
             total_string = ", ".join(total_strings)
             self._gui.get_object("fact_totals").set_text(total_string)
@@ -594,19 +460,6 @@ class HamsterApplet(object):
         """main window display and positioning"""
         self.button.set_active(is_active)
 
-        if self.last_activity and self.last_activity["end_time"] is None:
-            label = self.last_activity['name']
-            if self.last_activity['category'] != _("Unsorted"):
-                label += "@%s" %  self.last_activity['category']
-            self.activity_combo.child.set_text(label)
-
-            self.activity_combo.child.select_region(0, -1)
-            self._gui.get_object("more_info_label").hide()
-        else:
-            self.activity_combo.child.set_text('')
-            self._gui.get_object("more_info_label").show()
-
-
         if not is_active:
             self.window.hide()
             return
@@ -638,120 +491,65 @@ class HamsterApplet(object):
         self.window.unstick()
         self.window.stick() #show on all desktops
 
+        self.new_name.set_text("");
+        self.new_tags.set_text("");
         gobject.idle_add(self._delayed_display)  
         
     def _delayed_display(self):
         """show window only when gtk has become idle. otherwise we get
         mixed results. TODO - this looks like a hack though"""
         self.window.present()
-        self.activity_combo.grab_focus()
+        self.new_name.grab_focus()
         
 
     """events"""
 
-    def on_today_release_event(self, tree, event):
-        # a hackish solution to make edit icon keyboard accessible
-        pointer = event.window.get_pointer() # x, y, flags
-        path = tree.get_path_at_pos(pointer[0], pointer[1]) #column, innerx, innery
-        
-        if path and path[1] == self.edit_column:
-            self._open_edit_activity()
-            return True
-        
-        return False
         
     def on_toggle(self, widget):
         self.__show_toggle(None, self.button.get_active())
 
-    def on_activity_list_key_pressed(self, entry, event):
-        #tab will trigger going through autocomplete values when there are any
-        if event.keyval == gtk.keysyms.Tab:
-            event.keyval = gtk.keysyms.Down
-        
-        #down will trigger showing dropdown instead of selecting first whatever
-        elif event.keyval == gtk.keysyms.Down and len(self.completion.get_model()) == 0:
-            self.activity_combo.popup()
-            return True
-        
-        return False
-    
-    def on_activity_entry_changed(self, entry):
-        self.redo_autocomplete()
-        return False
-        
-    def on_activity_switched(self, component):
-        # do stuff only if user has selected something
-        # for other cases activity_edited will be triggered
-        if component.get_active_iter():
-            component.child.activate() # forward
-        return True
-
-    def on_activity_entered(self, component):
-        """fires, when user writes activity by hand"""
-        activity_name = component.get_text().decode('utf8', 'replace')
-        
-        if activity_name == "":
-            return
-        
-        runtime.storage.add_fact(activity_name)
-        runtime.dispatcher.dispatch('panel_visible', False)
-
     """listview events"""
     def on_todays_keys(self, tree, event):
         if (event.keyval == gtk.keysyms.Delete):
             self.delete_selected()
             return True
-        elif (event.keyval == gtk.keysyms.e  \
-              and event.state & gtk.gdk.CONTROL_MASK):
-            self._open_edit_activity()
-            return True
             
         return False
     
-    def _open_edit_activity(self):
+    def _open_edit_activity(self, row, fact):
         """opens activity editor for selected row"""
-        selection = self.treeview.get_selection()
-        (model, iter) = selection.get_selected()
-        fact_id = model[iter][0]
-            
-        custom_fact = CustomFactController(self, None, fact_id)
-        custom_fact.show()        
-    
+        custom_fact = CustomFactController(self, None, fact["id"])
+        custom_fact.show()
+        
     def on_today_row_activated(self, tree, path, column):
-        if column == self.edit_column:
-            self._open_edit_activity()
-        else:
-            selection = tree.get_selection()
-            (model, iter) = selection.get_selected()
-            
-            fact = model[iter][7]
-            if fact:
-                activity = fact['name']
-                if fact['category']:
-                    activity = '%s@%s' % (activity, fact['category'])
+        selection = tree.get_selection()
+        (model, iter) = selection.get_selected()
+        
+        fact = model[iter][6]
+        if fact:
+            activity = fact['name']
+            if fact['category']:
+                activity = '%s@%s' % (activity, fact['category'])
 
-                if fact['description']:
-                    activity = "%s, %s" % (activity, fact['description'])
-                    
-                runtime.storage.add_fact(activity)
-                runtime.dispatcher.dispatch('panel_visible', False)
+            tags = fact["tags"]
+            if fact["description"]:
+                tags.append(fact["description"])
+                
+            runtime.storage.add_fact(activity, ", ".join(tags))
+            runtime.dispatcher.dispatch('panel_visible', False)
         
         
     def on_windows_keys(self, tree, event_key):
         if (event_key.keyval == gtk.keysyms.Escape
           or (event_key.keyval == gtk.keysyms.w 
               and event_key.state & gtk.gdk.CONTROL_MASK)):
-            runtime.dispatcher.dispatch('panel_visible', False)
-            return True
+            if self.new_name.popup.get_property("visible") == False \
+               and self.new_tags.popup.get_property("visible") == False:
+                runtime.dispatcher.dispatch('panel_visible', False)
+                return True
         return False
         
     """button events"""
-    def on_stop_tracking(self, button):
-        runtime.storage.touch_fact(self.last_activity)
-        self.last_activity = None
-        self.update_label()
-        runtime.dispatcher.dispatch('panel_visible', False)
-
     def on_overview(self, menu_item):
         runtime.dispatcher.dispatch('panel_visible', False)
         stats_viewer = StatsViewer(self)
@@ -781,7 +579,7 @@ class HamsterApplet(object):
     
     """signals"""
     def after_activity_update(self, widget, renames):
-        self.refresh_dropdown()
+        self.new_name.refresh_activities()
         self.load_day()
         self.update_label()
     
@@ -803,6 +601,9 @@ class HamsterApplet(object):
             runtime.storage.touch_fact(self.last_activity,
                                        end_time = self.dbusIdleListener.getIdleFrom())
 
+    def on_day_start_changed(self, event, new_minutes):
+        self.day_start = self.config.get_day_start()
+        self.load_day()
 
     """global shortcuts"""
     def on_keybinding_activated(self, event, data):
@@ -828,14 +629,14 @@ class HamsterApplet(object):
             self.destroy()
 
         message_dialog = gtk.MessageDialog(buttons = gtk.BUTTONS_OK)
-        message_dialog.set_property("title", _("What to type in the activity box?"))
+        message_dialog.set_property("title", _("What should be typed in the activity box?"))
         message_dialog.connect("response", on_response)
         
-        more_info = _("""There is simple syntax that enables you to add details to your activities:
+        more_info = _("""There is a simple syntax that enables you to add details to your activities:
         
-"@" symbol marks category. Example: "watering flowers@home" will start tracking activity "watering flowers" in category "home".
+"@" symbol marks a category. Example: "watering flowers@home" will start tracking the activity "watering flowers" in the category "home".
 
-Comma (",") marks beginning of description. Example: "watering flowers, begonias and forgetmenots" will start tracking activity "watering flowers" and add description "begonias and forgetmenots" to it.
+Commas (",") mark beginning of a description. Example: "watering flowers, begonias and forgetmenots" will start tracking the activity "watering flowers" and add the description "begonias and forgetmenots" to it.
 
 Both can be combined: "watering flowers@home, begonias and forgetmenots" will work just fine!
 
@@ -845,3 +646,23 @@ Now, start tracking!
         message_dialog.set_markup(more_info)
         message_dialog.show()
 
+    def on_activity_text_changed(self, widget):
+        self.get_widget("switch_activity").set_sensitive(widget.get_text() != "")
+
+    def on_switch_activity_clicked(self, widget):
+        runtime.storage.add_fact(self.new_name.get_text().decode("utf8", "replace"),
+                                 self.new_tags.get_text().decode("utf8", "replace"))
+        self.new_name.set_text("")
+        self.new_tags.set_text("")
+        runtime.dispatcher.dispatch('panel_visible', False)
+
+    def on_stop_tracking_clicked(self, widget):
+        runtime.storage.touch_fact(runtime.storage.get_last_activity())
+        self.last_activity = None
+        runtime.dispatcher.dispatch('panel_visible', False)
+
+    def show(self):
+        self.window.show_all()
+        
+    def get_widget(self, name):
+        return self._gui.get_object(name)
