@@ -33,6 +33,44 @@ import re
 import locale
 import os
 
+def format_duration(minutes, human = True):
+    """formats duration in a human readable format.
+    accepts either minutes or timedelta"""
+    
+    if isinstance(minutes, dt.timedelta):
+        minutes = duration_minutes(minutes)
+        
+    if not minutes:
+        if human:
+            return ""
+        else:
+            return "00:00"
+    
+    hours = minutes / 60
+    minutes = minutes % 60
+    formatted_duration = ""
+    
+    if human:
+        if minutes % 60 == 0:
+            # duration in round hours
+            formatted_duration += _("%dh") % (hours)
+        elif hours == 0:
+            # duration less than hour
+            formatted_duration += _("%dmin") % (minutes % 60.0)
+        else:
+            # x hours, y minutes
+            formatted_duration += _("%dh %dmin") % (hours, minutes % 60)
+    else:
+        formatted_duration += "%02d:%02d" % (hours, minutes)
+    
+    
+    return formatted_duration
+
+def duration_minutes(duration):
+    """returns minutes from duration, otherwise we keep bashing in same math"""
+    return duration.seconds / 60 + duration.days * 24 * 60
+
+
 def load_ui_file(name):
     from configuration import runtime
     ui = gtk.Builder()
@@ -97,70 +135,13 @@ def format_activity(name, category, description, pad_description = False):
     if description:
         text+= "\n"
         if pad_description:
-            text += "          "
+            text += " " * 23
 
         text += """<span style="italic" size="small">%s</span>""" % description
         
     return text
     
 
-class ActivityColumn(gtk.TreeViewColumn):
-    def activity_painter(self, column, cell, model, iter):
-        activity_name = model.get_value(iter, self.name)
-        description = model.get_value(iter, self.description)
-        category = model.get_value(iter, self.category)
-        
-        markup = format_activity(activity_name, category, description)            
-        cell.set_property('markup', markup)
-        return
-        
-    def __init__(self, name, description, category = None):
-        gtk.TreeViewColumn.__init__(self)
-        
-        self.name, self.description, self.category = name, description, category
-        self.set_expand(True)
-        cell = gtk.CellRendererText()
-        self.pack_start(cell, True)
-        cell.set_property("ellipsize", pango.ELLIPSIZE_END)
-        self.set_cell_data_func(cell, self.activity_painter)
-
-def duration_minutes(duration):
-    """returns minutes from duration, otherwise we keep bashing in same math"""
-    return duration.seconds / 60 + duration.days * 24 * 60
-    
-def format_duration(minutes, human = True):
-    """formats duration in a human readable format.
-    accepts either minutes or timedelta"""
-    
-    if isinstance(minutes, dt.timedelta):
-        minutes = duration_minutes(minutes)
-        
-    if not minutes:
-        if human:
-            return ""
-        else:
-            return "00:00"
-    
-    hours = minutes / 60
-    minutes = minutes % 60
-    formatted_duration = ""
-    
-    if human:
-        if minutes % 60 == 0:
-            # duration in round hours
-            formatted_duration += _("%dh") % (hours)
-        elif hours == 0:
-            # duration less than hour
-            formatted_duration += _("%dmin") % (minutes % 60.0)
-        else:
-            # x hours, y minutes
-            formatted_duration += _("%dh %dmin") % (hours, minutes % 60)
-    else:
-        formatted_duration += "%02d:%02d" % (hours, minutes)
-    
-    
-    return formatted_duration
-
 def totals(iter, keyfunc, sumfunc):
     """groups items by field described in keyfunc and counts totals using value
        from sumfunc
@@ -217,7 +198,7 @@ def escape_pango(text):
     return text
 
 def figure_time(str_time):
-    if not str_time:
+    if not str_time or not str_time.strip():
         return None
     
     # strip everything non-numeric and consider hours to be first number
@@ -278,11 +259,6 @@ def parse_activity_input(text):
         elif res.start_time:
             text = text[text.find(" ")+1:]
     
-    #see if we have description of activity somewhere here (delimited by comma)
-    if text.find(",") > 0:
-        text, res.description = text.split(",", 1)
-        res.description = res.description.strip()
-
     if text.find("@") > 0:
         text, res.category_name = text.split("@", 1)
         res.category_name = res.category_name.strip()
